#!/usr/bin/ruby

def outmode(mode)
  #    disp hsyncstart hsyncend total
  #    doublescan -hsync +hsync -vsync +vsync
  
  # Example:
  #   timings 39722 48 16 33 10 96 2
  #
  #  -> 640 656 752 800  480 490 492 525
  #
  #      x x+16 x+16+96 x+16+96+48
  #      y y+10 y+10+2  y+10+2+33
  
  print 'Modeline '
  
  printf '%-17s', '"'+mode['name']+'"'
  
  printf ' %.3f', 1e6/mode['clock']
  printf '  %d %d %d %d', mode['visx'],
                         mode['visx']+mode['r'],
                         mode['visx']+mode['r']+mode['h'],
                         mode['visx']+mode['r']+mode['h']+mode['l']
  printf '  %d %d %d %d', mode['visy'],
                         mode['visy']+mode['b'],
                         mode['visy']+mode['b']+mode['v'],
                         mode['visy']+mode['b']+mode['v']+mode['t']
  
  print ' doublescan' if mode['double']
  print ' interlace'  if mode['laced']
  
  print mode['hsync'] ? ' +hsync' : ' -hsync'
  print mode['vsync'] ? ' +vsync' : ' -vsync'
  puts
end

mode = {}
while line=gets
  line.gsub! "\n", ''
  line.gsub! (/ *#.*/, '')
  line.gsub! (/^ */, '')
  line.gsub! (/ *$/, '')
  next if line.size == 0
  
  tokens = line.split(' ')
  case tokens[0]
    when 'mode'
      mode['name'] = tokens[1].gsub(/"/, '')
    when 'geometry'
      mode['visx']  = tokens[1].to_i
      mode['visy']  = tokens[2].to_i
      mode['virtx'] = tokens[3].to_i
      mode['virty'] = tokens[4].to_i
      mode['depth'] = tokens[5].to_i
    when 'timings' 
      mode['clock'] = tokens[1].to_i  # 1e6 / pixclock
      mode['l']     = tokens[2].to_i  # xlen-hsyncend
      mode['r']     = tokens[3].to_i  # hsyncstart-xres
      mode['t']     = tokens[4].to_i  # ylen-vsyncend
      mode['b']     = tokens[5].to_i  # vsyncstart-yres
      mode['h']     = tokens[6].to_i  # hsyncend-hsyncstart
      mode['v']     = tokens[7].to_i  # vsyncend-vsyncstart
    when 'hsync'
      mode['hsync'] = tokens[1]=='high'
    when 'vsync'
      mode['vsync'] = tokens[1]=='high'
    when 'double'
      mode['double']= tokens[1]=='high'
    when 'accel'
      mode['accel'] = tokens[1]=='high'
    when 'laced'
      mode['laced'] = tokens[1]=='high'
    when 'endmode'
      outmode mode
      mode = {}
  end
end
